﻿using Microsoft.AspNetCore.SignalR;
using System.Globalization;
using System.IO;
using System.Reflection.Metadata;
using System.Text;
using System.Text.RegularExpressions;
using TypeCoreLibrary;
using TypeValidationLibrary;


using UglyToad.PdfPig;
using static System.Net.Mime.MediaTypeNames;


static string ExtractValue(string source, string startPattern, string endPattern)
{
    if (string.IsNullOrEmpty(source) || string.IsNullOrEmpty(startPattern) || string.IsNullOrEmpty(endPattern))
    {
        return string.Empty;
    }

    int startIndex = source.IndexOf(startPattern);
    if (startIndex == -1)
    {
        return string.Empty; // no startPattern
    }

    startIndex += startPattern.Length; // move index after startPattern

    int endIndex = source.IndexOf(endPattern, startIndex);
    if (endIndex == -1)
    {
        return string.Empty; // no endPattern
    }

    return source.Substring(startIndex, endIndex - startIndex);
}

/*
== MBET summary ==
                                                     Slope =         129.563
                                                 Intercept =          -1.615e-01
                                Correlation coefficient, r =           0.999923
                                                 C constant=        -801.202
 
                                              Surface Area =          26.913 ~s/g
*/

(double CorrelationR, double SurfaceArea) = ParseMbetData(@"C:\RUB\!WORK\INF_TRR_CRC247\A08_NicolasCosanne\!NEW\HS-SA01-cal_report_01.txt");
Console.WriteLine(CorrelationR);
Console.WriteLine(SurfaceArea);
static (double CorrelationR, double SurfaceArea) ParseMbetData(string filePath)
{
    using var reader = new StreamReader(filePath);
    return ParseFromStream(reader);
}

static (double CorrelationR, double SurfaceArea) ParseMbetDatfa(Stream stream)
{
    using var reader = new StreamReader(stream);
    return ParseFromStream(reader);
}

static (double CorrelationR, double SurfaceArea) ParseFromStream(StreamReader reader)
{
    double? correlationR = null;
    double? surfaceArea = null;

    string? line;
    var numberStyle = NumberStyles.Float | NumberStyles.AllowThousands;
    var formatProvider = CultureInfo.InvariantCulture;

    while ((line = reader.ReadLine()) != null)
    {
        if (correlationR == null)
        {
            var matchR = Regex.Match(line, @"Correlation coefficient, r\s*=\s*([+-]?\d*\.?\d+(?:[eE][+-]?\d+)?)");
            if (matchR.Success)
            {
                correlationR = double.Parse(matchR.Groups[1].Value, numberStyle, formatProvider);
                continue;
            }
        }

        if (surfaceArea == null)
        {
            var matchSA = Regex.Match(line, @"Surface Area\s*=\s*([+-]?\d*\.?\d+(?:[eE][+-]?\d+)?)");
            if (matchSA.Success)
            {
                surfaceArea = double.Parse(matchSA.Groups[1].Value, numberStyle, formatProvider);
                continue;
            }
        }

        if (correlationR.HasValue && surfaceArea.HasValue)
            break;
    }

    if (correlationR == null || surfaceArea == null)
        throw new InvalidDataException("Не удалось найти требуемые параметры в файле.");

    return (correlationR.Value, surfaceArea.Value);
}

/*
// Protected BET adsorption report (pdf)
using (PdfDocument pdf = PdfDocument.Open(@"C:\RUB\!WORK\INF_TRR_CRC247\C02_LeonMueller\AC008 HT_LMueller_(Report - N2 BET).pdf"))
{
    //foreach (var page in pdf.GetPages())
    //{
    //    Console.WriteLine($"=== page{page.Number} ===");
    //    string text = page.Text;
    //    Console.WriteLine(text);
    //}


    var page = pdf.GetPages().FirstOrDefault();
    string rSt = null, areaSt = null;
    double r=0, area=0;
    string text = page.Text;
    if (page.Number == 1 && text.StartsWith("Anton Paar Kaomi for Nova") && text.Contains("Multipoint BET Summary/Results")) {
        // Isotherm BranchIsotherm BranchAdsorptionSlopeSlope42.65InterceptIntercept0.25184Correlation coeff., rCorrelation coeff., r0.999978C constantC constant170.353Surface areaSurface area81.174 m?/g
        rSt = ExtractValue(text, "Correlation coeff., rCorrelation coeff., r", "C constantC constant");
        areaSt = ExtractValue(text, "Surface areaSurface area", "m²/g");
        double.TryParse(rSt, out r);
        double.TryParse(areaSt, out area);
    }
    Console.WriteLine($"r = {r}");
    Console.WriteLine($"area = {area}");
}
*/

/*
string dirPath = @"C:\RUB\!WORK\COMPACT\chem. composition\_ALL";
DirectoryInfo dir = new DirectoryInfo(dirPath);

DateTime dt1 = DateTime.Now, dt2;
IFileValidator validator = new TypeValidator_EDX_CSV();
ITableGetter tableGetter = (ITableGetter)validator;
var files = dir.GetFiles();
var ok = 0;
var failExtension = 0;
var fail = 0;
int rows, containsCoordinatesCount=0;
bool containsCoordinates;
StringBuilder sb = new StringBuilder();
StringBuilder sbCsv = new StringBuilder();
sbCsv.AppendLine($"N;FileName;Rows;ContainsCoordinates;Code;Message");


for (var i = 0; i < files.Length; i++)
{
    FileInfo file = files[i];
    //Console.WriteLine(file.Name);
    validator.File = new FileInfo(file.FullName);
    var res = validator.Validate();
    //rows = tableGetter.DataTable?.Rows.Count ?? 0;
    //containsCoordinates = tableGetter.GetCoordinates;
    Table table = tableGetter.GetTable();
    rows = table?.DataTable?.Rows.Count ?? 0;
    containsCoordinates = table?.Coordinates ?? false;
    containsCoordinatesCount += containsCoordinates ? 1 : 0;
    if (res)
    {
        ok++;
    }
    else
    {
        if (res.Code == 1)
            failExtension++;
        else
            fail++;
    }
    sb.AppendLine($"{i + 1}) {file.Name} {res}");
    sbCsv.AppendLine($"{i + 1};{file.Name};{rows};{(containsCoordinates ? 1 : 0)};{res.Code};{res}");
}

dt2 = DateTime.Now;
sb.AppendLine($"=== TOTAL({ok+fail+failExtension}): ok={ok}, fail={fail}; failExtension={failExtension} === ContainsCoordinatesCount: {containsCoordinatesCount}. Completed in {(dt2-dt1).TotalMilliseconds} ms");
Console.WriteLine(sb.ToString());

File.WriteAllText("report.csv", sbCsv.ToString());   
*/