﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ViewEngines;
using System.Data.SqlClient;
using System.Data;
using System.Diagnostics;
using Dapper;
using System.Reflection;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using InfProject.DBContext;
using InfProject.Models;
using InfProject.Utils;
using WebUtilsLib;
using System.Linq;
using static System.Runtime.InteropServices.JavaScript.JSType;
using System.Collections.Generic;
using TypeValidationLibrary;
using System;
using System.Text;

namespace InfProject.Controllers;

[Authorize(Roles = "User,PowerUser,Administrator")]
public class DatasetController : Controller
{
    private readonly ILogger<DatasetController> logger;
    private readonly IConfiguration config;
    private readonly DataContext dataContext;
    private Encoding enc = Encoding.UTF8;

    public DatasetController(ILogger<DatasetController> logger, IConfiguration config, DataContext dataContext)
    {
        this.logger = logger;
        this.config = config;
        this.dataContext = dataContext;
    }

    /// <summary>
    /// Returns JSON dataset for a sample (Materials Library), that contatain all Measurement Areas, their Composition and all properties available
    /// /dataset/samplejson/54075
    /// </summary>
    /// <param name="objectId">SampleInfo.SampleId</param>
    /// <returns>table in JSON</returns>
    [HttpGet]
    public async Task<IActionResult> SampleJson([FromRoute(Name="id")] int objectId)
    {
        // await dataContext.ObjectInfo_GetVirtual<Sample>(idObject)).ToSampleFull(dataContext)
        Sample o = await dataContext.ObjectInfo_GetVirtual<Sample>(objectId);
        if (o == null || o.ObjectId == 0)
        {
            return NotFound($"Can not find object by objectId: {objectId}");
        }
        if (HttpContext.IsReadDenied(o.AccessControl, (int)o._createdBy))
        {
            return this.RedirectToLoginOrAccessDenied(HttpContext);
        }
        List<dynamic> data = await dataContext.Dataset_SampleMeasurementAreas(objectId, o.Elements);
        return Json(data);
        //return File(enc.GetBytes(str), "text/json", $"{objectId}_SampleDataset.json");

    }

    /// <summary>
    /// Returns CSV dataset for a sample (Materials Library), that contatain all Measurement Areas, their Composition and all properties available
    /// /dataset/samplecsv/54075
    /// </summary>
    /// <param name="objectId">SampleInfo.SampleId</param>
    /// <returns>table in CSV</returns>
    [HttpGet]
    public async Task<IActionResult> SampleCsv([FromRoute(Name = "id")] int objectId)
    {
        // await dataContext.ObjectInfo_GetVirtual<Sample>(idObject)).ToSampleFull(dataContext)
        Sample o = await dataContext.ObjectInfo_GetVirtual<Sample>(objectId);
        if (o == null || o.ObjectId == 0)
        {
            return NotFound($"Can not find object by objectId: {objectId}");
        }
        if (HttpContext.IsReadDenied(o.AccessControl, (int)o._createdBy))
        {
            return this.RedirectToLoginOrAccessDenied(HttpContext);
        }
        List<dynamic> data = await dataContext.Dataset_SampleMeasurementAreas(objectId, o.Elements);
        string str = DataTable2CSV.GetCSVfromList(separator: ";", sourceList: data, includeHeaders: true, changeEndL2Space: true);
        return File(enc.GetBytes(str), "text/csv", $"{objectId}_SampleDataset.csv");
    }


}
