﻿using InfProject.Models;
using TypeCoreLibrary;

namespace InfProject.DBContext;

/// <summary>
/// Extension methods for DataContext
/// </summary>
public static class DataContextExtension
{


    // TODO: implement context stuffing
    /// <summary>
    /// Extract Context for the current object (currentContextObject) from database having dataContext
    /// </summary>
    /// <param name="dataContext">DataContext - active database link</param>
    /// <param name="precursor">ContextPrecursor with lyambda and current object / parent objects</param>
    /// <returns></returns>
    public static async Task<Context?> ExtractContext(this DataContext dataContext, ContextPrecursor? precursor)
    // <param name="currentContextObject">current object (that should know "parent" sample)</param>
    // <param name="validationContext">validation Context (according TypeInfo.SettingsJson.ValidationContext)</param>
    //  ObjectInfo currentContextObject, string validationContext=null
    {
        Context? ctx = null;
        if (string.CompareOrdinal(precursor?.ValidationContext, "ChemicalSystem") == 0/* && precursor.GetContextAsync != null*/)
        {
            // ctx.ChemicalSystem = new string[] { "As", "Ga" };
            // ctx = precursor.GetContextAsync(precursor).GetAwaiter().GetResult();
            
            HashSet<string> set = new HashSet<string>();    // unique chemical elements
            List<string> list = null;
            if (precursor.ParentObjectIds != null && precursor.ParentObjectIds.Length > 0) {    // parents are specified
                // find all parents, that contain records in a Sample tanle to grab all chemical systems
                list = await dataContext.GetList_ExecDevelopment<string>($"select [Elements] from dbo.[Sample] WHERE SampleId IN ({string.Join(',', precursor.ParentObjectIds)})");
            }
            else {
                int objectId = (precursor.CurrentObject as ObjectInfo)?.ObjectId ?? precursor.ObjectId;
                if (objectId == 0)  // current object is not defined (Exception?)
                    return ctx;

                // find all parents, that contain records in a Sample table to grab all chemical systems
                list = await dataContext.GetList_ExecDevelopment<string>($@"select [Elements] from dbo.[Sample] as S
INNER JOIN dbo.ObjectLinkObject as OLO ON OLO.ObjectId=S.SampleId
WHERE OLO.LinkedObjectId=@objectId", new { objectId });
            }

            if (list != null) {
                foreach (var item in list)
                {
                    Array.ForEach(item.Split('-', StringSplitOptions.RemoveEmptyEntries), x => set.Add(x));
                }
                ctx = new Context()
                {
                    ChemicalSystem = set.ToArray(),
                    Version = 1
                };
            }
        }
        //Dictionary<string, string> dict = new Dictionary<string, string>();
        return ctx;
    }

}
