﻿using System.Data;
using Microsoft.Data.SqlClient;
using Dapper;
using InfProject.DTO;
using InfProject.Models;

namespace InfProject.DBContext;

public partial class DataContext
{
    /// <summary>
    /// General insert / update post-processing (so far required only for Samples)
    /// </summary>
    /// <param name="mode"></param>
    /// <param name="obj">object to change</param>
    /// <returns></returns>
    public async Task FinaliseOnObjectInsertUpdate(string mode, ObjectInfo obj) {

        // Tenant.SettingsJson == { "FinaliseOnObjectInsertUpdate": "SampleNamePrefix" }
        if (Tenant.GetJsonSettingsString("FinaliseOnObjectInsertUpdate") == "SampleNamePrefix") {
            // hardcode for a sample
            if (string.CompareOrdinal(mode, "insert") == 0 && obj.TypeId == SampleTypeId)
            { // can be called also on update (no change will be made for the second time)
              // Finalisation for the Sample:
                await FinalizeSampleName(obj, namePrefix: $"{obj.ExternalId} ");
            }
        }
    }


    /// <summary>
    /// Finalizes sample name by changing an ObjectName with adding a specified prefix
    /// </summary>
    /// <param name="obj">object to change</param>
    /// <param name="namePrefix">some string value to be prefix in ObjectName</param>
    /// <returns>Task</returns>
    private async Task FinalizeSampleName(ObjectInfo obj, string namePrefix)
    {
        if (string.IsNullOrEmpty(namePrefix?.Trim()))
            return;
        int objectId = obj.ObjectId;    // objectId to change
        using (IDbConnection connection = new SqlConnection(ConnectionString))
        {
            int rowsAffected = await connection.ExecuteAsync("UPDATE dbo.ObjectInfo SET ObjectName=@namePrefix + ObjectName WHERE TenantId=@TenantId AND ObjectId=@objectId and LEFT(ObjectName, LEN(@namePrefix))<>@namePrefix", 
                new { TenantId, objectId, namePrefix });
            if (rowsAffected > 0) {
                obj.ObjectName = namePrefix + obj.ObjectName;   // required to adjust current object (so that information is actual for notifications)
            }
        }
    }



}
