﻿using System.Data;
using Microsoft.Data.SqlClient;
using Dapper;
using static Dapper.SqlMapper;
using Microsoft.IdentityModel.Protocols;
using WebUtilsLib;
using System.Text;
using System.Diagnostics.Metrics;
using InfProject.Models;

namespace InfProject.DBContext;

public partial class DataContext
{

    /// <summary>
    /// Returns dataset for a sample (Materials Library), that contatain all Measurement Areas, their Composition and all properties available
    /// tested on ObjectId=54075
    /// </summary>
    /// <param name="objectId">SampleInfo.SampleId</param>
    /// <param name="elementsDashSeparated">chemical system in the sample (retrieved on the previous step)</param>
    /// <returns>JSON (List&lt;dynamic&gt;)</returns>
    public async Task<List<dynamic>> Dataset_SampleMeasurementAreas(int objectId, string elementsDashSeparated)
    {
        string[] elements = elementsDashSeparated.Split('-', StringSplitOptions.RemoveEmptyEntries);
        string elementsCSV = string.Join(',', elements);
        string addCompositionColumns_c = string.Join(',', elements.Select(el => $"C.[Element_{el}]"));
        string addCompositionColumns_as = string.Join(',', elements.Select(el => $"[{el}] as [Element_{el}]"));


        DynamicParameters parameters = new DynamicParameters();
        parameters.Add("TenantId", TenantId);
        parameters.Add("objectId", objectId);
        parameters.Add("CompositionTypeId", CompositionTypeId);


        // retrieve properties
        bool is342MA = false, useInnerJoin = false;
        List<(string type, string propertyName, int count, int sortCode)> properties = await Property_GetAllPropertiesCountForMeasurementAreasInSample(objectId, filterTypeId: CompositionTypeId);
        if (properties.Count>0 && properties[0].type == "Int" && properties[0].propertyName == "Measurement Area" && properties[0].count == 342)
        {
            is342MA = useInnerJoin = true;
        }
        StringBuilder dbPropertiesName = new StringBuilder();
        StringBuilder dbPropertiesJoin = new StringBuilder();
        for (int i = 0; i < properties.Count; i++)
        {
            (string type, string propertyName, int count, int sortCode) = properties[i];
            // TODO: дописать!!!
            parameters.Add($"PropertyName{i}", propertyName);
            //dbPropertiesName.AppendLine($", P{i}.[Value] as [@PropertyName{i}]");
            dbPropertiesName.AppendLine($", P{i}.[Value] as [{DBUtils.Prepare4SQL(propertyName).Replace("[", string.Empty).Replace("]", string.Empty)}]");
            if (useInnerJoin && count == 342)   // inner joins are faster, so prefer them, unless we can't affort it
            {
                dbPropertiesJoin.AppendLine($"INNER JOIN dbo.Property{type} as P{i} ON P{i}.ObjectId=C.SampleId and P{i}.PropertyName=@PropertyName{i}");
            }
            else {
                useInnerJoin = false;
                dbPropertiesJoin.AppendLine($"LEFT OUTER JOIN dbo.Property{type} as P{i} ON P{i}.ObjectId=C.SampleId and P{i}.PropertyName=@PropertyName{i}");
            }
        }

        using (IDbConnection connection = new SqlConnection(ConnectionString))
        {
            string sql = $@"
SELECT C.SampleId as CompositionId, {addCompositionColumns_c} 
{dbPropertiesName.ToString()}
FROM dbo.vObjectLinkObject as v
INNER JOIN (select SampleId, {addCompositionColumns_as} from (
	select SampleId, ElementName, ValuePercent from dbo.Composition 
	WHERE SampleId IN (select ChildObjectId from dbo.vObjectLinkObject WHERE ParentObjectId=@ObjectId and ChildTypeId=@CompositionTypeId)	-- 1026 rows (3xComposition)
) as t
PIVOT (SUM(ValuePercent) FOR ElementName IN ({elementsCSV})) AS pivotedsales) as C ON v.ChildObjectId=C.SampleId
-- Add other properties
{dbPropertiesJoin.ToString()}
WHERE v.ParentTenantId=@TenantId AND v.ParentObjectId=@ObjectId AND v.ChildTenantId=@TenantId AND v.ChildTypeId=@CompositionTypeId	-- 342 rows
ORDER BY {(is342MA ? "[Measurement Area]" : "CompositionId")}";
            var res = await connection.QueryAsync<dynamic>(sql, parameters);

            // GetTable_ExecDevelopment

            return res?.ToList();
        }
    }

}
