﻿using Newtonsoft.Json;
using System.Diagnostics.CodeAnalysis;

namespace InfProject.Models
{
    /// <summary>
    /// description of the graog node for Cytoscape.js
    /// </summary>
    /// <param name="id">node idenifier</param>
    /// <param name="name">node name</param>
    /// <param name="typeId">type identifier</param>
    /// <param name="bucket">bucket identifier</param>
    /// <param name="parentId">parent node identifier</param>
    public class GraphNode(string id, string name, int typeId, string bucket, string? parentId=null) : IEqualityComparer<GraphNode>
    {
        /// <summary>
        /// node idenifier
        /// </summary>
        public string Id { get; init; } = id;

        /// <summary>
        /// node name
        /// </summary>
        public string Name { get; init; } = name;

        /// <summary>
        /// type identifier
        /// </summary>
        public int TypeId { get; init; } = typeId;

        /// <summary>
        /// node name
        /// </summary>
        public string Bucket { get; init; } = bucket;


        /// <summary>
        /// parent node identifier
        /// </summary>
        public string? ParentId { get; set; } = parentId;

        /// <summary>
        /// Equality comparer (uses id only!)
        /// </summary>
        /// <param name="x">GraphNode left</param>
        /// <param name="y">GraphNode right</param>
        /// <returns>true - objects are equal (id are equal)</returns>
        public bool Equals(GraphNode? x, GraphNode? y)
        {
            if (ReferenceEquals(x, y))
                return true;
            if (x is null || y is null)
                return false;
            return x.Id == y.Id;
        }

        public int GetHashCode([DisallowNull] GraphNode obj) => Id.GetHashCode();

        public object ToObject() => string.IsNullOrEmpty(ParentId) ?
                new { data = new { id = Id, label = Name, bucket = Bucket }, classes = $"{Bucket} t{TypeId}" } :
                new { data = new { id = Id, label = Name, bucket = Bucket, parent = ParentId }, classes = $"{Bucket} t{TypeId}" };

        public string ToJsonString() {
            // adds quotes to property names 
            return JsonConvert.SerializeObject(ToObject());
        }

        public static string ToJsonStringNoQuotes(object obj) {
            var serializer = new JsonSerializer();
            var stringWriter = new StringWriter();
            using (var writer = new JsonTextWriter(stringWriter))
            {
                //writer.Formatting = Formatting.Indented;
                writer.QuoteName = false;
                //writer.IndentChar = ' ';
                //writer.Indentation = 1;
                // writer.QuoteChar = '\'';
                serializer.Serialize(writer, obj);
            }
            var json = stringWriter.ToString();
            return json;
        }

        public string ToJsonStringNoQuotes() => ToJsonStringNoQuotes(ToObject());

    }
}
