﻿using System.ComponentModel.DataAnnotations;
using System.Diagnostics.CodeAnalysis;
using System.Net.Sockets;
using FluentValidation;
using InfProject.Utils;
using Newtonsoft.Json;

namespace InfProject.Models;

/// <summary>
/// for validation:
/// https://learn.microsoft.com/en-us/aspnet/core/mvc/models/validation
/// </summary>
public class ObjectInfoLinked : ObjectInfo
{
    public int ObjectLinkObjectId { get; set; }
    public int MainObjectId { get; set; }
    public int SortCodeLink { get; set; }


    public DateTime? Link_created { get; set; }

    public int? Link_createdBy { get; set; }

    public DateTime? Link_updated { get; set; }

    public int? Link_updatedBy { get; set; }

    // Link type characterization - April 2024 - begin
    public int? LinkTypeObjectId { get; set; }
    public string? LinkTypeObjectName { get; set; }
    public string? LinkTypeObjectNameUrl { get; set; }
    // Link type characterization - April 2024 - end

    public override string ToString() => $"Link for {base.ToString()} [ObjectLinkObjectId={ObjectLinkObjectId}, MainObjectId={MainObjectId}, LinkTypeObjectId={LinkTypeObjectId}, LinkType={LinkTypeObjectName}]";

    public object ToParentObject(int objectId) => 
        new { data = new { id = $"olo{ObjectLinkObjectId}_o{ObjectId}_o{objectId}", source = $"o{ObjectId}", target = $"o{objectId}", label = (LinkTypeObjectName ?? string.Empty) } };

    public object ToChildObject(int objectId) =>
        new { data = new { id = $"olo{ObjectLinkObjectId}_o{objectId}_o{ObjectId}", source = $"o{objectId}", target = $"o{ObjectId}", label = (LinkTypeObjectName ?? string.Empty) } };

    public string ToJsonParentString(int objectId) =>
        JsonConvert.SerializeObject(ToParentObject(objectId));
        // $"{{\"data\": {{ \"id\": \"olo{ObjectLinkObjectId}_o{ObjectId}_o{objectId}\", \"source\": \"o{ObjectId}\", \"target\": \"o{objectId}\", \"label\": \"{LinkTypeObjectName}\" }}}}";
    public string ToJsonChildString(int objectId) =>
        // $"{{\"data\": {{ \"id\": \"olo{ObjectLinkObjectId}_o{objectId}_o{ObjectId}\", \"source\": \"o{objectId}\", \"target\": \"o{ObjectId}\", \"label\": \"{LinkTypeObjectName}\" }}}}";
        JsonConvert.SerializeObject(ToChildObject(objectId));

    public string ToJsonParentStringNoQuotes(int objectId) => GraphNode.ToJsonStringNoQuotes(ToParentObject(objectId));

    public string ToJsonChildStringNoQuotes(int objectId) => GraphNode.ToJsonStringNoQuotes(ToChildObject(objectId));

}
