﻿using InfProject.DBContext;
using InfProject.Models;
using System.IO;
using System.Text;
using static WebUtilsLib.DBUtils;
using WebUtilsLib;
using InfProject.DTO;
using static Microsoft.EntityFrameworkCore.DbLoggerCategory;
using Microsoft.VisualStudio.Web.CodeGenerators.Mvc.Templates.Blazor;
using Microsoft.AspNetCore.Http.HttpResults;
using Microsoft.Extensions.Primitives;
using Microsoft.CodeAnalysis.CSharp.Syntax;
using TypeCoreLibrary;

namespace InfProject.Utils
{
    public class PropertiesUtils
    {
        public class PropertyForOutput {
            public int PropertyId;
            public string path;
            public int level; 
            public bool isSeparapor;
            public int countFilledChildren;
            public string value;
            public string nameNormalized;
        }

        public static PropertyForOutput[] GetPropertiesDesctiptionForOutput(List<dynamic> listAll) {
            PropertyForOutput[] desc = new PropertyForOutput[listAll.Count];
            string path;
            int level;
            for (var i = 0; i < listAll.Count; i++) {   // First Iteration: fill all the array
                path = listAll[i].PropertyName.Replace(" => ", "}");    // replace " => " with "}"
                level = path.Length - path.Replace("}", string.Empty).Length;   // level(determined as number of '}'): 0, 1, 2, ...
                string[] arrNames = (listAll[i].PropertyName ?? string.Empty).Split(" => ", StringSplitOptions.RemoveEmptyEntries);
                desc[i] = new PropertyForOutput() { 
                    PropertyId = listAll[i].PropertyId ?? listAll[i].TemplatePropertyId,
                    path = path,
                    level = level,
                    isSeparapor = listAll[i].TemplateComment == "SEPARATOR",
                    countFilledChildren = 0,
                    value = listAll[i].Value,
                    nameNormalized = arrNames[arrNames.Length - 1]
                };
            }
            for(var i = 0; i < listAll.Count; i++){   // Second Iteration: calculate countFilledChildren
                if (desc[i].isSeparapor)
                {
                    desc[i].countFilledChildren = desc.Where(x => !x.isSeparapor && x.path.StartsWith(desc[i].path + "}") && (
                        !string.IsNullOrEmpty(x.value)
                    )).Count();
                }
            }
            return desc;
        }


        /// <summary>
        /// Updates object properties based on HttpRequest form and a template
        /// </summary>
        /// <param name="objectId">object identifier</param>
        /// <param name="typeId">type identifier</param>
        /// <param name="httpContext">HttpHttpContext to take data from a request (html form is used) and get user from context</param>
        /// <param name="dataContext">DataContext to work with DB</param>
        /// <param name="objectFormPrefix">Form prefix for the object (required on batch operations to distinguish between several forms placed together for several objects), see also pEditItem_PropertiesForm.cshtml</param>
        /// <returns>(int updatedInserted, // updatedInserted properties count, 
        /// int deleted, // deleted properties count
        /// string log) // detailed log as a string</returns>
        /// <exception cref="Exception">Exception if no access to the object from the current user context</exception>
        public static async Task<(int updatedInserted, int deleted, string log)> UpdateInsert_PropertiesFromForm(int objectId, int typeId, HttpContext httpContext, DataContext dataContext, string objectFormPrefix = null)
        {
            int updatedInsertedLinks=0, deletedLinks = 0;   // for ObjectLinkObject
            if (objectFormPrefix == null) 
                objectFormPrefix = string.Empty;
            ObjectInfo objOld = await httpContext.GetObjectAndCheckWriteAccess(dataContext, objectId);   // Exception if no access
            HttpRequest Request = httpContext.Request;
            int userId = httpContext.GetUserId();
            DateTime dt = DateTime.Now.AddMilliseconds(-100);   // just to make sure, than nothing is deleted unattended on quick updates with the same time (TODO: sync times with SQL server - could be a problems if SQL Server in on other VM with non-synchronized time)
            StringBuilder sb = new StringBuilder();
            int updatedInserted = 0;

            // read template to know preperties restrictions and validation
            List<PropertyValue> templateObject = await dataContext.Property_GetTemplatePropertiesForType(typeId);

            foreach (string item in Request.Form[$"{objectFormPrefix}prop_templ"])
            {
                int i = item.IndexOf('_');
                if (i < 0)
                {
                    throw new Exception($"Underscore is not found in prop_templ value: \"{item}\"");
                }
                int.TryParse(item.Substring(0, i), out int PropertyId);   // existing PropertyId
                int.TryParse(item.Substring(i + 1, item.Length - i - 1), out int TemplatePropertyId);    // Template PropertyId

                string name = Request.Form[$"{objectFormPrefix}nam" + item][0] ?? string.Empty;
                string? sortStr = Request.Form[$"{objectFormPrefix}sor" + item][0];
                string? typeStr = Request.Form[$"{objectFormPrefix}typ" + item][0];
                string? valueForm = Request.Form[$"{objectFormPrefix}val" + item][0];
                string? epsilonStr = Request.Form[$"{objectFormPrefix}eps" + item].ToString(); // may not exist
                string? comment = Request.Form[$"{objectFormPrefix}com" + item][0];

                // This is a new feature to support Substrate in a Sample (which is a linked object) in a common template
                string? proplink = null;
                StringValues vp = Request.Form[$"{objectFormPrefix}proplink{item}"];
                if (vp.Count > 0)
                {
                    proplink = vp[0];   // if value=="LinkObject" - a link should be created intead of writing to Property* table
                }

                // Source ObjectId (optional field)
                string? sourceObjectIdStr = string.IsNullOrEmpty(Request.Form[$"{objectFormPrefix}soid" + item]) ? string.Empty : Request.Form[$"{objectFormPrefix}soid" + item][0];

                sb.AppendLine($"{(!string.IsNullOrEmpty(proplink) ? $"{proplink}: " : string.Empty)}PropertyId={PropertyId}; TemplatePropertyId={TemplatePropertyId}. type={typeStr}, value={valueForm}, epsilon={epsilonStr}, comment={comment}");
                // UPDATE / INSERT in database
                //newObject = await dataContext.ObjectInfo_UpdateInsert(obj);

                // templateObject, TemplatePropertyId, valueForm

                // find a property from Template
                PropertyValue? propTemplate = templateObject.Find(x => x.PropertyId == TemplatePropertyId);

                // throws an exception if not valid
                UpdateInsert_PropertiesFromForm_Validate(name, valueForm, propTemplate);

                if (!string.IsNullOrEmpty(valueForm))
                {
                    PropertyType type = (PropertyType)Enum.Parse(typeof(PropertyType), typeStr);
                    int.TryParse(sortStr, out int sortCode);
                    int.TryParse(sourceObjectIdStr, out int sourceObjectId);
                    if (proplink == "LinkObject" && type == PropertyType.Int)   // int
                    {
                        // links from ObjectLinkObject table
                        string? sproplinkRubricId = Request.Form[$"{objectFormPrefix}proplink{item}rubricid"][0];
                        if (int.TryParse(sproplinkRubricId, out int proplinkRubricId) && proplinkRubricId != 0
                            && int.TryParse(valueForm, out int linkedObjectId)/* && linkedObjectId != 0*/)
                        {
                            // previous value
                            // <input type="hidden" name="@($"{objectFormPrefix}proplink{props[i].PropertyId}_{props[i].TemplatePropertyId}prev")" value="@linkedObjectId">
                            string? spropPrevValue = Request.Form[$"{objectFormPrefix}proplink{item}prev"][0];
                            if (spropPrevValue != linkedObjectId.ToString()) // change => do it in db
                            {
                                string sql = $@"DELETE FROM dbo.ObjectLinkObject WHERE ObjectId=@ObjectId AND LinkedObjectId IN (select ObjectId from dbo.ObjectInfo where TenantId=@TenantId AND RubricId=@RubricId)
if @LinkedObjectId!=0 and exists(select ObjectId from dbo.ObjectInfo where TenantId=@TenantId AND RubricId=@RubricId AND ObjectId=@LinkedObjectId)
begin
    INSERT INTO dbo.ObjectLinkObject(ObjectId, LinkedObjectId, SortCode, _created, _createdBy, _updated, _updatedBy, LinkTypeObjectId)
        VALUES(@ObjectId, @LinkedObjectId, @SortCode, @_created, @userId, @_created, @userId, NULLIF(@LinkTypeObjectId, 0))
end";
                                await dataContext.Execute(sql, new
                                {
                                    dataContext.TenantId,
                                    RubricId = proplinkRubricId,
                                    ObjectId = objectId,
                                    LinkedObjectId = linkedObjectId,
                                    SortCode = sortCode,
                                    _created = dt,
                                    userId,
                                    LinkTypeObjectId = sourceObjectId
                                });
                            }
                        }
                    }
                    else
                    {
                        // properties from Property* table
                        double? epsilon = null;
                        if (double.TryParse(epsilonStr, out double eps))
                        {
                            epsilon = eps;
                        }
                        object val = valueForm;
                        if (type == PropertyType.Float)
                        {
                            double.TryParse(valueForm, out double v);
                            val = v;
                        }
                        if (type == PropertyType.Int)
                        {
                            long.TryParse(valueForm, out long v);
                            val = v;
                        }
                        PropertyValue value = new PropertyValue()
                        {
                            PropertyId = PropertyId,
                            PropertyType = type,
                            PropertyName = name,
                            Value = val,
                            ValueEpsilon = epsilon,
                            SortCode = sortCode,
                            Row = null,
                            Comment = comment,
                            SourceObjectId = sourceObjectId
                        };
                        int newPropertyId = await dataContext.Property_UpdateInsert(value, dt, userId, objectId);
                        updatedInserted++;
                    }
                }
            }


            // clean up previous properties by date 
            int deleted = await dataContext.Property_DeleteNonTableBefore(objectId, dt);
            return (updatedInserted, deleted, log: sb.ToString());
        }

        /// <summary>
        /// validation of a property according to template
        /// ok - no exceprion; invalid - exception
        /// </summary>
        /// <param name="name">name of the property (from form)</param>
        /// <param name="valueForm">value of the property (from form)</param>
        /// <param name="propTemplate">PropertyValue - template property (to extract validator and required)</param>
        /// <exception cref="Exception"></exception>
        public static void UpdateInsert_PropertiesFromForm_Validate(string name, string? valueForm, PropertyValue? propTemplate)
        {
            // check required condition
            if ((propTemplate?.CommentObject?.IsRequired ?? false) && string.IsNullOrEmpty(valueForm))
            {
                throw new Exception($"The value must be set [{propTemplate.PropertyName}]");
            }

            // validation required
            if (!string.IsNullOrEmpty(propTemplate?.CommentObject?.ValidationSchema))
            {
                // create validator
                IFileValidator validator = TypeInfoExtension.GetValidator(propTemplate?.CommentObject?.ValidationSchema);
                validator.Stream = new MemoryStream(Encoding.UTF8.GetBytes(valueForm)) { Position = 0 };
                TypeValidatorResult validatorRes = validator.Validate();
                if (!validatorRes)
                {
                    throw new Exception($"Error validating {propTemplate?.PropertyName} ({name}): {validatorRes.Message}");
                }
            }
        }
    }
}
