﻿var graphSettings = {};
graphSettings.style = [ // the stylesheet for the graph
    {
        selector: 'node',
        style: {
            'background-color': '#999',
            'label': 'data(label)' // 'label': 'data(id)'
        }
    },


    {   // Substrate
        selector: 'node.t5',
        style: {
            'background-color': '#DCECD9',
        }
    },
    {   // Sample
        selector: 'node.t6',
        style: {
            'background-color': '#B2D4EC',
            'shape': 'rectangle',
        }
    },
    {   // Composition
        selector: 'node.t8',
        style: {
            'background-color': '#B2FFFD',
        }
    },
    {   // Synthesis
        selector: 'node.t18',
        style: {
            'background-color': '#A8FF96',
        }
    },
    {   // Request for Synthesis
        selector: 'node.t83',
        style: {
            'background-color': '#FFE699',
        }
    },
    {   // Idea or Plan
        selector: 'node.t89',
        style: {
            'background-color': '#FFFF99',
        }
    },
    {   // Calculation/Computational Composition
        selector: 'node.t93',
        style: {
            'background-color': '#FFCCCC',
        }
    },
    {   // Report
        selector: 'node.t96',
        style: {
            'background-color': '#28E74F',
        }
    },


    {
        selector: 'node.CORE',
        style: {
            'shape': 'round-rectangle',
        }
    },
    {
        selector: 'edge',
        style: {
            'label': 'data(label)',
            'width': 3,
            'line-color': '#ccc',
            'target-arrow-color': '#ccc',
            'target-arrow-shape': 'triangle',
            'curve-style': 'bezier'
        }
    },
    {
        selector: ':parent',
        style: {
            'background-opacity': 0.3,
            'border-color': '#2B65EC'
        }
    },

    {
        selector: 'node:selected',
        style: {
            //'background-color': '#F08080',
            'shape': 'star',
            'border-color': 'red',
            'border-width': '1px'
        }
    },
    {
        selector: 'edge:selected',
        style: {
            'line-color': 'red',
        }
    }
];

// show information for node
async function getGraphNodeInfo(obj, evt) {
    console.log(obj);
    console.log(evt);
    const nodeId = obj.id();
    const id = nodeId.substring(1);
    if ($("#curdesc").attr("data-id") == id)
        return;
    let objData = await $.ajax({
        async: true,
        type: "GET",
        url: "/ajax/getobject",
        data: { "objectId": id },
        dataType: "json"
    });
    let htmlAdds = "";
    try {
        htmlAdds = "<h3>" + objData[0].ObjectName + "</h3>";
        if (objData[0].ObjectDescription) {
            htmlAdds += "<div>" + objData[0].ObjectDescription + "</div>";
        }
        htmlAdds += "<hr>";
    }
    catch (err) { }
    const json = JSON.stringify(objData, undefined, 2);
    $("#curdesc").attr("data-id", id).html(htmlAdds + `<pre>${json}</pre>`);
}

app.graphExtendIds = [];
// extend graph to the node (add all adgacent objects and links)
async function extendGraphFromNode(obj, evt) {
    console.log("extendGraphFromNode 0");
    console.log(obj);
    console.log(evt);
    console.log("extendGraphFromNode 1");
    const nodeId = obj.id();
    let id = nodeId.substring(1);
    let temp, newNodes;
    console.log("extendGraphFromNode 2. id = " + id);
    if (app.graphExtendIds.includes(id)) {
        console.log("extendGraphFromNode includes " + id + " BYE!");
        return;
    }
    console.log("extendGraphFromNode 3");
    let objData = await $.ajax({
        async: true,
        type: "GET",
        url: "/ajax/getadjacentnodesandlinks",
        data: { "objectId": id },
        dataType: "json"
    });
    console.log(`extendGraphFromNode 4 [length = ${objData.length}]`);
    console.log(objData);

    // lock existing nodes and their positions
    cy.nodes(':unlocked').forEach(node => node.lock());
    let nodes2Add = [];

    for (let i = 0; i < objData.length; i++) {
        id = objData[i].data.id;
        temp = cy.getElementById(id);
        console.log(`check(${id}): ${temp}`);
        if (temp && temp.length > 0) {  // found 
console.log("EXISTS: " + JSON.stringify(objData[i]));
        }
        else {    // https://js.cytoscape.org/#cy.getElementById == not found
            // https://js.cytoscape.org/#cy.add
            nodes2Add.push(objData[i]);
console.log("ADDED: " + JSON.stringify(objData[i]));
        }
    }

    if (nodes2Add.length > 0) {
        newNodes = cy.add(nodes2Add);
    }


    // https://js.cytoscape.org/#cy.add
    // graphElements.includes()
    // let eles = cy.add(objData);
    console.log("extendGraphFromNode pre-refreshLayout");
    refreshLayout(newNodes);

    cy.nodes(':locked').forEach(node => node.unlock());

    return objData;
}

// https://stackoverflow.com/questions/55003657/cytoscape-js-dynamically-add-nodes-without-moving-the-others
function refreshLayout(newNodes) {
    //cy.layout(myLayoutOptions).run();
    //cy.layout.stop();
    //cy.layout = cy.elements().makeLayout(layoutOpts);
    newNodes.layout({
        name: 'cose', animate: false,
        refresh: .1, edgeElasticity: 20, fit: false, gravity: 100
    }).run();
}