﻿using System.Text.Json.Nodes;
using System.Text.Json;
using Microsoft.Extensions.Configuration;

namespace TypeValidationLibrary;

public class WaferSearchOnSpreadsheetWrapper {
    private readonly IConfiguration config;
    public WaferSearchSettings Settings { get; set; }

    public WaferSearchOnSpreadsheetWrapper(IConfiguration config)
    {
        this.config = config;
        Settings = ReadConfig(config);
    }

    /// <summary>
    /// Configuration read
    /// </summary>
    /// <param name="config"></param>
    /// <returns></returns>
    public static WaferSearchSettings ReadConfig(IConfiguration config)
    {
        IConfigurationSection section = config.GetSection("GoogleSheets:Credentials");
        JsonNode? jsonNode = WebUtilsLib.ConfigurationUtils.Serialize(section);
        WaferSearchSettings settings = new WaferSearchSettings()
        {
            CredentialsJson = jsonNode?.ToJsonString(new JsonSerializerOptions() { WriteIndented = true }),
            SpreadsheetId = config["GoogleSheets:SpreadsheetId"],
            // string[] keys = { "Lasermark", "Wafer", "EXTRAS", "Company", "Invoice Date" };
            // "SpreadsheetKeyNamesFilter": "Lasermark,Wafer,EXTRAS,Company,Invoice Date"
            Keys = config["GoogleSheets:SpreadsheetKeyNamesFilter"]?.Split(',', StringSplitOptions.RemoveEmptyEntries | StringSplitOptions.TrimEntries)
        };
        return settings;
    }

    /// <summary>
    /// Search for a specific wafer LaserMark in the Google spreadsheet (as defined in the constructor)
    /// </summary>
    /// <param name="waferLaserMark">wafer LaserMark to search for</param>
    /// <param name="comparator">Func&lt;string&gt; with all key-values pair from the row matched</returns>
    /// <exception cref="Exception">exceptions if no access or wrong document format</exception>
    public async Task<List<KeyValuePair<string, string>>> Search(string waferLaserMark, Func<string, string, bool>? comparator = null)
    {
        WaferSearchOnSpreadsheet searcher = new WaferSearchOnSpreadsheet(Settings);
        Dictionary<string, string> result = await searcher.Search(waferLaserMark, comparator);
        List<KeyValuePair<string, string>> filteredResult = result?.Where(x => Settings.Keys.Length < 1 || Settings.Keys.Contains(x.Key)).ToList();
        return filteredResult;
    }
}
