using Microsoft.AspNetCore.Mvc;
using System.Net.Mime;
using System.Data;
using TypeCoreLibrary;
using static WebUtilsLib.DBUtils;

namespace WebApiTypeServiceTemplateCSharp.Controllers;

/// <summary>
/// Gets data from the document
/// </summary>
[ApiController]
[Route("[controller]")]
public class DataController : ControllerBase
{
    private readonly ILogger<DataController> _logger;

    public DataController(ILogger<DataController> logger)
    {
        _logger = logger;
    }


    /// <summary>
    /// Incoming stream processing to get DatabaseValues (post call to /data/DatabaseValuesBody)
    /// </summary>
    /// <param name="array">binary request body (POST)</param>
    /// <returns>CoordinatesResult, specifying coordinates</returns>
    [HttpPost("databasevaluesbody")]
    [Consumes(MediaTypeNames.Application.Octet)] // application/octet-stream
    [Produces(typeof(DatabaseValues))]
    public DatabaseValues DatabaseValuesBody([FromBody] byte[] array)
    {
        (Stream inpuStream, string? filename) = Utilities.GetDataFromRequest(Request);
        // validation
        TypeValidatorResult vr = Utilities.Validate(inpuStream, filename);
        if (!vr)
            throw new Exception(vr.Message);

        DatabaseValues databaseValues = new DatabaseValues();
        // TODO: fill the object with data to return
        return databaseValues;
    }



    /// <summary>
    /// Incoming stream processing to get DataTable (post call to /data/TableBody)
    /// </summary>
    /// <param name="array">binary request body (POST)</param>
    /// <returns>CoordinatesResult, specifying coordinates</returns>
    [HttpPost("tablebody")]
    [Consumes(MediaTypeNames.Application.Octet)] // application/octet-stream
    [Produces(typeof(Table))]
    public Table TableBody([FromBody] byte[] array)
    {
        (Stream inpuStream, string? filename) = Utilities.GetDataFromRequest(Request);
        // validation
        TypeValidatorResult vr = Utilities.Validate(inpuStream, filename);
        if (!vr)
            throw new Exception(vr.Message);

        DataTable dataTable = new DataTable();
        // TODO: fill the table with data

        CoordinatesResult coordinates = new CoordinatesResult();
        // TODO: fill the coordinates 

        DependenciesResult dependencies = new DependenciesResult();
        // TODO: fill the dependencies 

        Table table = new Table(dataTable, coordinates, dependencies);
        return table;
    }
}
