using Microsoft.AspNetCore.Mvc;
using System.ComponentModel.DataAnnotations;
using System.Net.Mime;
using TypeCoreLibrary;

namespace WebApiTypeServiceTemplateCSharp.Controllers;

[ApiController]
[Route("[controller]")]
public class ValidationController : ControllerBase
{
    private readonly ILogger<ValidationController> _logger;

    public ValidationController(ILogger<ValidationController> logger)
    {
        _logger = logger;
    }

    /// <summary>
    /// Validation of incoming stream from POST body (post call to /EDX/Validation/body)
    /// </summary>
    /// <param name="array">binary request body (POST)</param>
    /// <returns>TypeValidatorResult, specifying validation success</returns>
    [HttpPost("body")]
    [Consumes(MediaTypeNames.Application.Octet)] // application/octet-stream
    [Produces(typeof(TypeValidatorResult))]
    public TypeValidatorResult Body([FromBody] byte[] array)
    {
        (Stream inpuStream, string? filename) = Utilities.GetDataFromRequest(Request);

        // validation
        TypeValidatorResult vr = Utilities.Validate(inpuStream, filename);
        return vr;
    }
}
