using TypeCoreLibrary;

namespace WebApiTypeServiceTemplateCSharp;

public class Utilities
{
    /// <summary>
    /// Reads the data from request
    /// </summary>
    /// <param name="request">HttpRequest (POST)</param>
    /// <returns>(Stream inpuStream, string? filename) = input data, file name</returns>
    public static (Stream inpuStream, string? filename) GetDataFromRequest(HttpRequest request)
    {
        request.EnableBuffering();  // important, otherwise can't seek input stream
        Stream inpuStream = request.Body;  // binary content of the document will be here
        if (inpuStream.CanSeek)
        {
            // Reset the position to zero to read from the beginning.
            inpuStream.Seek(0, System.IO.SeekOrigin.Begin);
        }

        // extract the filename from the header (to analyze it if neccesary)
        string? filename = WebUtilsLib.HttpUtils.ExtractFileNameFromHeader(request);

        return (inpuStream, filename);
    }


    /// <summary>
    /// Common validation for a type
    /// </summary>
    /// <param name="inpuStream">file data</param>
    /// <param name="filename">file name</param>
    /// <returns>TypeValidatorResult</returns>
    public static TypeValidatorResult Validate(Stream inpuStream, string? filename) {
        // Here you have a document to validate according to your User Defined Type (UDT)
        // 1) Analyse the file name, if required: string? filename
        // 2) Analyse the file contents: Stream req
        // 3) return TypeValidatorResult object
        TypeValidatorResult vr = new TypeValidatorResult();   // success
        // TypeValidatorResult vrFail = new TypeValidatorResult(500, "Validation error: explanation"); // failure
        return vr;
    }

}
