from fastapi import FastAPI, Request, Header
from fastapi.responses import HTMLResponse
    
app = FastAPI(docs_url="/")

@app.post(
    "/validation/body",
    openapi_extra={
        "requestBody": {
            "content": {
                "application/octet-stream": {
                    "schema": {
                        "type": "array",
                        
                    }
                }
            }
        }
    },
)
async def validation_of_incoming_file(request: Request):
	# read the incoming document 
	byte_data = await request.body()

	# processing...

	# return the validation status (Failure)
	# return { "Code": 500, "Message": "Reject reason", "Warning": None }
	# return the validation status (Success)
	return { "Code": 0, "Message": "", "Warning": "" }


@app.post(
    "/data/databasevaluesbody",
    openapi_extra={
        "requestBody": {
            "content": {
                "application/octet-stream": {
                    "schema": {
                        "type": "array",
                        
                    }
                }
            }
        }
    },
)
async def Incoming_stream_processing_to_get_databasevalues(request: Request):
	# read the incoming document 
	byte_data = await request.body()
	# processing...
	# return the result for writing properties info database
	return {
  "DeletePreviousCompositions": False,
  "Compositions": [
    {
      "CompositionElements": [
        {
          "CompoundIndex": 0,
          "ElementName": "Ag",
          "ValueAbsolute": None,
          "ValuePercent": 1.488411059559923
        },
        {
          "CompoundIndex": 10,
          "ElementName": "Pd",
          "ValueAbsolute": None,
          "ValuePercent": 40.10507479349179
        },
        {
          "CompoundIndex": 20,
          "ElementName": "Pt",
          "ValueAbsolute": None,
          "ValuePercent": 58.40651414694828
        }
      ],
      "DeletePreviousProperties": True,
      "Properties": [
        {
          "PropertyId": 0,
          "Type": 2,
          "Name": "Measurement Area",
          "Value": 1,
          "ValueEpsilon": None,
          "SortCode": 0,
          "Row": None,
          "Comment": None,
          "SourceObjectId": None
        },
        {
          "PropertyId": 0,
          "Type": 1,
          "Name": "x",
          "Value": -22.5,
          "ValueEpsilon": None,
          "SortCode": 0,
          "Row": None,
          "Comment": None,
          "SourceObjectId": None
        },
        {
          "PropertyId": 0,
          "Type": 1,
          "Name": "y",
          "Value": -40.5,
          "ValueEpsilon": None,
          "SortCode": 0,
          "Row": None,
          "Comment": None,
          "SourceObjectId": None
        }
      ]
    },
    {
      "CompositionElements": [
        {
          "CompoundIndex": 0,
          "ElementName": "Ag",
          "ValueAbsolute": None,
          "ValuePercent": 68.27554906235781
        },
        {
          "CompoundIndex": 10,
          "ElementName": "Pd",
          "ValueAbsolute": None,
          "ValuePercent": 0
        },
        {
          "CompoundIndex": 20,
          "ElementName": "Pt",
          "ValueAbsolute": None,
          "ValuePercent": 31.724450937642185
        }
      ],
      "DeletePreviousProperties": True,
      "Properties": [
        {
          "PropertyId": 0,
          "Type": 2,
          "Name": "Measurement Area",
          "Value": 342,
          "ValueEpsilon": None,
          "SortCode": 0,
          "Row": None,
          "Comment": None,
          "SourceObjectId": None
        },
        {
          "PropertyId": 0,
          "Type": 1,
          "Name": "x",
          "Value": 13.5,
          "ValueEpsilon": None,
          "SortCode": 0,
          "Row": None,
          "Comment": None,
          "SourceObjectId": None
        },
        {
          "PropertyId": 0,
          "Type": 1,
          "Name": "y",
          "Value": 45,
          "ValueEpsilon": None,
          "SortCode": 0,
          "Row": None,
          "Comment": None,
          "SourceObjectId": None
        }
      ]
    }
  ],
  "CompositionsForSampleUpdate": [],
  "DeletePreviousProperties": True,
  "Properties": [
    {
      "PropertyId": 0,
      "Type": 1,
      "Name": "Ag",
      "Value": 1.4884110540151596,
      "ValueEpsilon": None,
      "SortCode": 10,
      "Row": 1,
      "Comment": "Minimal Ag content",
      "SourceObjectId": None
    },
    {
      "PropertyId": 0,
      "Type": 1,
      "Name": "Ag",
      "Value": 69.68466639518738,
      "ValueEpsilon": None,
      "SortCode": 10,
      "Row": 2,
      "Comment": "Maximal Ag content",
      "SourceObjectId": None
    }
  ]
}


@app.post(
    "/data/tablebody",
    openapi_extra={
        "requestBody": {
            "content": {
                "application/octet-stream": {
                    "schema": {
                        "type": "array",
                        
                    }
                }
            }
        }
    },
)
async def Incoming_stream_processing_to_get_DataTable(request: Request):
	# read the incoming document 
	byte_data = await request.body()
	# processing...
	# return the result for writing properties info database
	return { 
		"DataTable": [ 
			{ "Index": 1, "x": -22.5, "y": -40.5, "Ag": 1.4884111, "Pd": 40.105075, "Pt": 58.406514 },
			{ "Index": 342, "x": 13.5, "y": 45, "Ag": 68.27555, "Pd": 0, "Pt": 31.72445 }
			],
		"Coordinates": { "CoordinateNames": [ "x", "y" ], "HasCoordinates": True },
		"Dependencies": { "DependencyNames": [ "Index", "x", "y" ], "HasDependencies": True }
	}


@app.post(
    "/visualisation",
    openapi_extra={
        "requestBody": {
            "content": {
                "application/octet-stream": {
                    "schema": {
                        "type": "array",
                        
                    }
                }
            }
        }
    },
)
async def Incoming_stream_processing_to_get_Html(request: Request, filename: str = Header(None)):
	# read the incoming document 
	byte_data = await request.body()
	body_length = len(byte_data)
	# processing...
	# return the result in HTML (could contain JS/CSS)
	html_content = f"<html><body>Put visualisation for accepted data ({body_length} bytes; filename={filename}) here</body></html>"
	return HTMLResponse(content=html_content)
