﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using System.ComponentModel.DataAnnotations;
using System.Data;
using System.IO;
using System.Net.Mime;
using TypeCoreLibrary;
using TypeValidationLibrary;
using static WebUtilsLib.DBUtils;

namespace WebApiValidator.Controllers;

/// <summary>
/// Controller to validate EDX documents
/// </summary>
[ApiController]
[Route("xrd/data")]
public class XRDDataController : ControllerBase
{
    private readonly ILogger<XRDDataController> logger;

    public XRDDataController(ILogger<XRDDataController> logger)
    {
        this.logger = logger;
    }


    #region DataTable

    /// <summary>
    /// Incoming stream processing to get DataTable (post call to /XRD/data/TableBody)
    /// </summary>
    /// <param name="array">binary request body (POST)</param>
    /// <returns>CoordinatesResult, specifying coordinates</returns>
    [HttpPost("tablebody")]
    [Consumes(MediaTypeNames.Application.Octet)] // application/octet-stream
    [Produces(typeof(Table))]
    public Table TableBody([FromBody] byte[] array)
    {
        Request.EnableBuffering();  // important, otherwise can't seek input stream
        Stream request = Request.Body;
        if (request.CanSeek)
        {
            // Reset the position to zero to read from the beginning.
            request.Seek(0, System.IO.SeekOrigin.Begin);
        }

        // extract the filename from the header (to analyze if neccesary)
        string? filename = WebUtilsLib.HttpUtils.ExtractFileNameFromHeader(Request);

        TypeValidator_XRD_CSV v = new TypeValidator_XRD_CSV();
        TypeValidatorResult validationResult = v.ValidateStream(request);
        if (validationResult) {
            Table table = v.GetTable();
            return table;
        }
        else
            throw new ArgumentException(validationResult.ToString());
    }

    #endregion // DataTable



    #region DatabaseValues

    /// <summary>
    /// Incoming stream processing to get DataTable (post call to /XRD/data/DatabaseValuesBody)
    /// </summary>
    /// <param name="array">binary request body (POST)</param>
    /// <returns>CoordinatesResult, specifying coordinates</returns>
    [HttpPost("databasevaluesbody")]
    [Consumes(MediaTypeNames.Application.Octet)] // application/octet-stream
    [Produces(typeof(DatabaseValues))]
    public DatabaseValues DatabaseValuesBody([FromBody] byte[] array)
    {
        Request.EnableBuffering();  // important, otherwise can't seek input stream
        Stream req = Request.Body;
        if (req.CanSeek)
        {
            // Reset the position to zero to read from the beginning.
            req.Seek(0, System.IO.SeekOrigin.Begin);
        }

        // extract the filename from the header (to analyze if neccesary)
        string? filename = WebUtilsLib.HttpUtils.ExtractFileNameFromHeader(Request);

        TypeValidator_XRD_CSV v = new TypeValidator_XRD_CSV();
        TypeValidatorResult validationResult = v.ValidateStream(req);
        if (validationResult)
            return v.GetDatabaseValues();
        else
            throw new ArgumentException(validationResult.ToString());
    }

    #endregion // DatabaseValues
}