﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;
using Microsoft.Extensions.Primitives;
using System.Net.Mime;
using TypeCoreLibrary;
using TypeValidationLibrary;

namespace WebApiValidator.Controllers;

/// <summary>
/// Controller to validate EDX documents
/// </summary>
[ApiController]
[Route("xrd/validation")]
public class XRDValidationController : ControllerBase
{
    private readonly ILogger<XRDValidationController> logger;

    public XRDValidationController(ILogger<XRDValidationController> logger)
    {
        this.logger = logger;
    }


    /// <summary>
    /// Validation of incoming stream from POST body (post call to /XRD/Validation/body)
    /// </summary>
    /// <param name="array">binary request body (POST)</param>
    /// <returns>TypeValidatorResult, specifying validation success</returns>
    [HttpPost("body")]
    [Consumes(MediaTypeNames.Application.Octet)] // application/octet-stream
    [Produces(typeof(TypeValidatorResult))]
    public TypeValidatorResult Body([FromBody] byte[] array) {
        Request.EnableBuffering();  // important, otherwise can't seek input stream
        Stream req = Request.Body;
        if (req.CanSeek)
        {
            // Reset the position to zero to read from the beginning.
            req.Seek(0, System.IO.SeekOrigin.Begin);
        }

        // extract the filename from the header (to analyze if neccesary)
        string? filename = WebUtilsLib.HttpUtils.ExtractFileNameFromHeader(Request);

        TypeValidator_XRD_CSV v = new TypeValidator_XRD_CSV();
        TypeValidatorResult vr = v.ValidateStream(req);
        return vr;
    }

}